from __future__ import annotations

import asyncio
import os
from dataclasses import dataclass
from string import Template
from typing import TYPE_CHECKING

import pytest

import tests.t_utils.common as t_common
import tests.t_utils.custom_models as t_custom
import tests.t_utils.entsoe_models as t_entsoe

if TYPE_CHECKING:
    import pandas as pd

    from cimsparql.data_models import BusDataFrame, ConnectivityNodeDataFrame

swing_bus_query = Template(
    """
PREFIX cim:<${cim}>
select ?top_node ?island
where {
    ?island cim:TopologicalIsland.AngleRefTopoligicalNode ?top_node
}
"""
)


@dataclass
class NodeConsistencyData:
    bus: BusDataFrame
    connectivity_nodes: ConnectivityNodeDataFrame
    two_node_dfs: dict[str, pd.DataFrame]
    single_node_dfs: dict[str, pd.DataFrame]
    swing_buses: pd.DataFrame


if TYPE_CHECKING:
    # Alias for a dict with node consistency data. The key should be the name
    # of the model
    from cimsparql.model import Model

    CONSISTENCY_DATA = dict[str, NodeConsistencyData | None]


async def get_node_consistency_test_data(
    model: Model | None,
) -> NodeConsistencyData | None:
    if model is None:
        return None
    loop = asyncio.get_event_loop()
    bus = await loop.run_in_executor(None, model.bus_data)
    con_nodes = await loop.run_in_executor(None, model.connectivity_nodes)

    # Results that has node_1 and node_2 which should be part of bus
    two_node_names = ["ac_lines", "series_compensators", "transformer_branches"]

    two_node_dfs = await asyncio.gather(
        loop.run_in_executor(None, model.ac_lines),
        loop.run_in_executor(None, model.series_compensators),
        loop.run_in_executor(None, model.transformer_branches),
    )

    # Results that has node which should be part of bus
    single_node_names = ["loads", "sync_machines", "converters"]
    single_node_dfs = await asyncio.gather(
        loop.run_in_executor(None, model.loads),
        loop.run_in_executor(None, model.synchronous_machines),
        loop.run_in_executor(None, model.converters),
    )
    swing_buses = await loop.run_in_executor(
        None, model.get_table_and_convert, model.template_to_query(swing_bus_query)
    )
    return NodeConsistencyData(
        bus,
        con_nodes,
        dict(zip(two_node_names, two_node_dfs, strict=True)),
        dict(zip(single_node_names, single_node_dfs, strict=True)),
        swing_buses,
    )


async def collect_node_consistency_data() -> list[NodeConsistencyData | None]:
    test_models = [t_custom.federated_model(), t_entsoe.micro_t1_nl()]
    if any(m.model is None and m.must_run_in_ci and os.getenv("CI") for m in test_models):
        pytest.fail("Model that must run in CI is None")
    return await asyncio.gather(*[get_node_consistency_test_data(m.model) for m in test_models])


@pytest.fixture(scope="session")
def nc_data() -> CONSISTENCY_DATA:
    """Return test data which in a datastructure suitable for consistency checks for node data."""
    res = asyncio.run(collect_node_consistency_data())
    return {"model": res[0]}


def skip_on_missing(data: CONSISTENCY_DATA | NodeConsistencyData | None, model_name: str):
    if not data:
        pytest.skip(f"No data collected for {model_name}")


@pytest.mark.parametrize("model_name", ["model"])
def test_node_consistency(nc_data: CONSISTENCY_DATA, model_name: str):
    data = nc_data[model_name]
    skip_on_missing(data, model_name)
    assert data

    mrids = set[str](data.connectivity_nodes.index)
    for name, df in data.two_node_dfs.items():
        msg = f"Error two node: {name}"
        assert set(df["connectivity_node_1"]).issubset(mrids), msg
        if name != "transformer_branches":
            # Connectivity node 2 is equal to the transformer mrid for transformer branches
            assert set(df["connectivity_node_2"]).issubset(mrids), msg

    for name, df in data.single_node_dfs.items():
        msg = f"Error single node: {name}"
        assert set(df["connectivity_node"]).issubset(mrids), msg


@pytest.mark.parametrize("model_name", ["model"])
def test_swing_bus_consistency(nc_data: CONSISTENCY_DATA, model_name: str):
    data = nc_data[model_name]
    skip_on_missing(data, model_name)
    assert data
    assert data.bus["is_swing_bus"].sum() == len(data.swing_buses)


@pytest.mark.parametrize("test_model", t_entsoe.micro_models() + t_custom.all_custom_models())
async def test_all_connectivity_nodes_fetched(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model

    query = Template(
        """
        PREFIX cim:<${cim}>
        PREFIX SN:<${SN}>

        select (count(distinct ?s) as ?count) where  {
            values ?true {True "true"}
            ?s ^cim:Terminal.ConnectivityNode/cim:Terminal.ConductingEquipment/SN:Equipment.networkAnalysisEnable ?true
        }
        """
    )
    count_query = test_model.model.template_to_query(query)
    num_nodes_df = test_model.model.get_table_and_convert(count_query)
    num_connectivity_nodes = int(num_nodes_df["count"].iloc[0])

    # Verify that we get some nodes
    assert num_connectivity_nodes > 0

    connectivity_nodes = test_model.model.connectivity_nodes()

    # Verify that we get information for all nodes in the model
    assert len(connectivity_nodes) == num_connectivity_nodes



from cimsparql.sparql_result_json import SparqlResultHead, SparqlResultValue


def test_populate_sparql_result_head_by_name():
    sparql_head_1 = SparqlResultHead.model_validate({"variables": ["a", "b"], "links": ["l1", "l2"]})

    sparql_head_2 = SparqlResultHead.model_validate({"variables": ["a", "b"], "links": ["l1", "l2"]})
    assert sparql_head_1 == sparql_head_2


def test_populate_sparql_result_value_by_name():
    sparql_value_1 = SparqlResultValue.model_validate({"type": "literal", "value": "value"})

    sparql_value_2 = SparqlResultValue.model_validate({"type": "literal", "value": "value"})
    assert sparql_value_1 == sparql_value_2



import contextlib
from copy import deepcopy
from string import Template
from typing import Any

import pytest

import tests.t_utils.common as t_common
import tests.t_utils.entsoe_models as t_entsoe
from cimsparql.graphdb import GraphDBClient
from cimsparql.model import SingleClientModel


@contextlib.contextmanager
def tmp_client(model: SingleClientModel, new_client: GraphDBClient):
    orig_client = deepcopy(model.client)
    try:
        model.client = new_client
        yield model
    finally:
        model.client = orig_client


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_bus_data_micro_t1_nl(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    model = test_model.model
    assert model
    bus_data = model.bus_data()

    # 11 topological nodes + 1 dummy node three windnig transformer
    # + 6 dummy nodes for two winding transformer
    assert len(bus_data) == 18
    assert bus_data.index.name == "node"


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_cim_version_micro_t1_nl(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    model = test_model.model
    assert model
    assert model.cim_version == 16


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_repo_not_empty_micro_t1_nl(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    assert not test_model.model.client.empty


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_cim_converters_micro_t1_nl(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    converters = test_model.model.converters()
    assert converters.empty


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_full_model_micro_t1_nl(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    full_model = test_model.model.full_model()

    profiles = {
        "http://entsoe.eu/CIM/SteadyStateHypothesis/1/1",
        "http://entsoe.eu/CIM/Topology/4/1",
        "http://entsoe.eu/CIM/StateVariables/4/1",
    }
    assert profiles.issubset(full_model["profile"])

    row = full_model.query("model == 'urn:uuid:10c3fda3-35b7-47b0-b3c6-919c3e82e974'").iloc[0]
    assert row["profile"] == "http://entsoe.eu/CIM/SteadyStateHypothesis/1/1"
    assert row["time"] == "2017-10-02T09:30:00Z"
    assert row["version"] == "3"


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_substation_voltage_level(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    voltage_level = test_model.model.substation_voltage_level()
    assert sorted(voltage_level["v"].tolist()) == [225.0, 380.0, 400.0]


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_branch_node_withdraw(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    withdraw = test_model.model.branch_node_withdraw()
    assert len(withdraw) == 29


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_loads(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    loads = test_model.model.loads()
    assert len(loads) == 6
    expected_names = {
        "BE-Load_1",
        "NL-Load_3",
        "L-1230804819",
        "BE-Load_2",
        "NL-Load_1",
        "NL-Load_2",
    }
    assert set(loads["name"]) == expected_names


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
@pytest.mark.parametrize(
    ("region", "expected_names"),
    [
        (".*", {"BE-G1", "NL-G2", "NL-G3", "NL-G1"}),
        ("NL", {"NL-G2", "NL-G3", "NL-G1"}),
        ("BE", {"BE-G1"}),
    ],
)
def test_sync_machines(test_model: t_common.ModelTest, region: str, expected_names: set[str]):
    t_common.check_model(test_model)
    assert test_model.model
    synchronous_machines = test_model.model.synchronous_machines(region)
    assert expected_names == set(synchronous_machines["name"])


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_connections(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    connections = test_model.model.connections()
    assert len(connections) == 26


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_ltc_fixed_angle_equals_zero(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    model = test_model.model
    branches = model.get_table_and_convert(model.transformer_branches_query(), index="mrid")
    angle = model.winding_angle()
    pytest.approx(branches.loc[angle.index, "angle"] == 0.0)


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_aclines(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    ac_lines = test_model.model.ac_lines(region=".*")
    assert len(ac_lines) == 2


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
@pytest.mark.parametrize(
    "params",
    [
        {
            "region": None,
            "num": 15,
            "end_num_count": [1, 7, 7],
            "expect_names": {
                "NL-TR2_1",
                "NL_TR2_2",
                "NL_TR2_3",
                "BE-TR2_1",
                "BE-TR2_2",
                "BE-TR2_3",
                "BE-TR3_1",
            },
        },
        {
            "region": "NL",
            "num": 6,
            "end_num_count": [3, 3],
            "expect_names": {"NL-TR2_1", "NL_TR2_2", "NL_TR2_3"},
        },
    ],
)
def test_transformers(test_model: t_common.ModelTest, params: dict[str, Any]):
    t_common.check_model(test_model)
    assert test_model.model
    transformers = test_model.model.transformers(params["region"])
    assert len(transformers) == params["num"]
    end_num_count = transformers["end_number"].value_counts()
    assert sorted(end_num_count) == params["end_num_count"]
    assert set(transformers["name"].unique()) == params["expect_names"]


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
@pytest.mark.parametrize(
    "params",
    [{"region": ".*", "num": 15, "num_transf": 7}, {"region": "NL", "num": 6, "num_transf": 3}],
)
def test_transformers_connectivity(test_model: t_common.ModelTest, params: dict[str, Any]):
    t_common.check_model(test_model)

    assert test_model.model
    transformers = test_model.model.transformers(params["region"])
    assert len(transformers) == params["num"]

    # Groupby mrid of the power transformer. Count gives the number of windings
    assert len(transformers.groupby(["p_mrid"]).count()) == params["num_transf"]


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_windings(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    transformer_branches = test_model.model.transformer_branches()
    expect_names = {
        "051cb1e0-2d34-3ac3-487e-a04c30781280": "NL_TR2_2",
        "1464d1bc-4a33-634c-7d93-b550dadf8b06": "NL-TR2_1",
        "bd9f254a-6448-10e5-657b-eb33b827887d": "NL-TR2_1",
        "f8ea6566-e73c-547d-a572-526e45334891": "NL_TR2_3",
        "91ec0b2c-2447-b8da-a141-90c0c35b33d6": "NL_TR2_2",
        "af598929-431f-3ef3-e636-87923c96f17c": "NL_TR2_3",
        "5f68a129-d5d8-4b71-9743-9ca2572ba26b": "BE-TR3_1",
        "49ca3fd4-1b54-4c5b-83fd-4dbd0f9fec9d": "BE-TR2_1",
        "81a18364-0397-48d3-b850-22a0e34b410f": "BE-TR2_2",
        "e1f661c0-971d-4ce5-ad39-0ec427f288ab": "BE-TR3_1",
        "664a19e1-1dc2-48d5-b265-c0630981e61c": "BE-TR2_2",
        "f58281c5-862a-465e-97ec-d809be6e24ab": "BE-TR2_3",
        "1912224a-9e98-41aa-84cf-00875bce7264": "BE-TR2_1",
        "2e21d1ef-2287-434c-a767-1ca807cf2478": "BE-TR3_1",
        "35651e25-a77a-46a1-92f4-443d6acce90e": "BE-TR2_3",
    }
    assert transformer_branches["name"].to_dict() == expect_names
    assert transformer_branches["status"].all()

    # On transformer branches, the mrid of node_2 should be a transformer
    transformers = test_model.model.transformers()
    assert set(transformer_branches["connectivity_node_2"]).issubset(set(transformers["p_mrid"]))


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
async def test_disconnected(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    disconnected = test_model.model.disconnected()
    assert len(disconnected) == 2

    # For convenience just test parts of mrids exists
    expected_partial_mrids = ["9f984b04", "f04ec73"]
    assert all(disconnected["mrid"].str.contains(x).any() for x in expected_partial_mrids)


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_powerflow(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    flow = test_model.model.powerflow()
    assert len(flow) == 24


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_regions(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    regions = test_model.model.regions()
    assert set(regions["region"]) == {"BE", "EU", "NL"}


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_coordinates(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    model = test_model.model
    crd = model.coordinates()
    assert set(crd["epsg"]), {"4326"}

    cim = model.client.prefixes["cim"]
    categories = {f"{cim}ACLineSegment", f"{cim}Substation"}
    assert set(crd["rdf_type"]).difference(categories) == set()

    assert len(crd) == 49
    coordinates = crd.astype({"x": float, "y": float})

    assert ((coordinates["x"] > 4.0) & (coordinates["x"] < 6.0)).all()
    assert ((coordinates["y"] > 50.0) & (coordinates["y"] < 53.0)).all()


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
async def test_not_empty_dc_active_flow(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    flow = test_model.model.dc_active_flow()
    assert not flow.empty


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_transformer_windings(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    transformer_windings = test_model.model.transformer_windings()

    assert len(transformer_windings) == 15


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_switches(test_model: t_common.ModelTest):
    t_common.check_model(test_model)
    assert test_model.model
    switches = test_model.model.switches()

    # 26 Breakers from CGMES_v2.4.15-MicroGridTestConfiguration_v2.docx (table 8)
    assert len(switches) == 26
    assert (switches["equipment_type"] == "Breaker").all()


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_sv_power_deviation(test_model: t_common.ModelTest) -> None:
    t_common.check_model(test_model)
    assert test_model.model

    deviation = test_model.model.sv_power_deviation()

    client = test_model.model.clients["Sv power deviation"]

    count_query = test_model.model.template_to_query(
        Template("""
        PREFIX cim: <$cim>
        select (count(distinct ?tp_node) as ?num) where {
        ?s cim:Terminal.TopologicalNode ?tp_node
        }
        """)
    )

    # Count number of tp nodes connected to a terminal
    num = int(client.exec_query(count_query).results.bindings[0]["num"].value)
    assert num > 0
    assert len(deviation) == num


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_base_voltage(test_model: t_common.ModelTest) -> None:
    t_common.check_model(test_model)
    assert test_model.model

    voltage = test_model.model.base_voltage()
    assert len(voltage) == 8


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_associated_switches(test_model: t_common.ModelTest) -> None:
    t_common.check_model(test_model)
    assert test_model.model

    associated_switches = test_model.model.associated_switches()
    assert not associated_switches.empty

    # Test values for power transformer BE-TR3. Should be connected to two switches
    switches = set(associated_switches.query("name == 'BE-TR2_1'")["switch_names"].iloc[0].split(","))
    assert switches == {"BE_Breaker_1", "BE_Breaker_2"}


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_gen_unit_and_sync_machine_mrid(test_model: t_common.ModelTest) -> None:
    t_common.check_model(test_model)
    assert test_model.model

    gen = test_model.model.gen_unit_and_sync_machine_mrid()
    assert len(gen) == 4


@pytest.mark.parametrize("test_model", t_entsoe.micro_models())
def test_station_group_for_power_unit(test_model: t_common.ModelTest) -> None:
    t_common.check_model(test_model)
    assert test_model.model

    group_for_units = test_model.model.station_group_for_power_unit()
    assert len(group_for_units) == 10



import json
from contextlib import AbstractContextManager, nullcontext
from http import HTTPStatus

import pytest
import tenacity
from pytest_httpserver import HTTPServer
from werkzeug import Request, Response

from cimsparql.graphdb import GraphDBClient, RestApi, ServiceConfig
from cimsparql.sparql_result_json import SparqlResultJsonFactory


class FailFirstOkSecondHandler:
    def __init__(self) -> None:
        self.num_calls = 0

    def request_handler(self, _: Request) -> Response:
        self.num_calls += 1
        if self.num_calls == 1:
            return Response(status=HTTPStatus.NOT_FOUND)
        result = SparqlResultJsonFactory.build()
        return Response(
            json.dumps(result.model_dump(mode="json", by_alias=True)),
            status=HTTPStatus.OK,
            mimetype="application/json",
        )


def fail_first_ok_second_server(httpserver: HTTPServer) -> str:
    handler = FailFirstOkSecondHandler()
    httpserver.expect_request("/sparql").respond_with_handler(handler.request_handler)
    return httpserver.url_for("/sparql")


@pytest.mark.parametrize(
    ("num_retries", "exception_context"), [(0, pytest.raises(tenacity.RetryError)), (1, nullcontext())]
)
def test_retry_sync(httpserver: HTTPServer, num_retries: int, exception_context: AbstractContextManager[None]):
    url = fail_first_ok_second_server(httpserver)

    config = ServiceConfig(
        server=url,
        rest_api=RestApi.DIRECT_SPARQL_ENDPOINT,
        retry_stop_criteria=tenacity.stop_after_attempt(num_retries + 1),
    )

    client = GraphDBClient(config)

    with exception_context:
        client.exec_query("select * where {?s ?p ?o}")



import functools
import logging
import os
import re
from pathlib import Path

import tests.t_utils.common as t_common
from cimsparql.adaptions import XmlModelAdaptor
from cimsparql.graphdb import GraphDBClient, RestApi, ServiceConfig, new_repo_blazegraph
from cimsparql.model import ModelConfig, SingleClientModel, get_federated_cim_model, get_single_client_model

this_dir = Path(__file__).parent

logger = logging.getLogger()


def micro_t1_nl_graphdb() -> t_common.ModelTest:
    repo = os.getenv("GRAPHDB_MICRO_NL_REPO", "abot-micro-nl")
    model = None
    try:
        s_cfg = ServiceConfig(repo=repo, max_delay_seconds=1)
        m_cfg = ModelConfig(system_state_repo=f"repository:{repo}", eq_repo=f"repository:{repo}")
        if os.getenv("GRAPHDB_SERVER"):
            model = get_single_client_model(s_cfg, m_cfg)
    except Exception:
        logger.exception("Failed to get single model")
    return t_common.ModelTest(model, must_run_in_ci=False, cleanup=False)


def split_tpsvssh(fname: Path) -> tuple[str, str]:
    tpsvssh_lines = list[str]()
    remaining_lines = list[str]()

    tpsvssh_contexts = {
        "http://entsoe.eu/CIM/StateVariables/4/1",
        "http://entsoe.eu/CIM/SteadyStateHypothesis/1/1",
        "http://entsoe.eu/CIM/Topology/4/1",
    }

    # Regex extracts the content between < > of the last occurence on each line
    prog = re.compile(r"<([^>]+)>[^>]+$")
    with fname.open() as infile:
        for line in infile:
            m = prog.search(line)
            if m and m.group(1) in tpsvssh_contexts:
                tpsvssh_lines.append(line)
            else:
                remaining_lines.append(line)
    return "".join(tpsvssh_lines), "".join(remaining_lines)


@functools.lru_cache
def federated_micro_t1() -> t_common.ModelTest:
    model = None
    url = t_common.rdf4j_url()
    try:
        tpsvssh_client = t_common.init_repo_rdf4j(url, "federated_micro_t1_nl_tpsvssh")
        eq_client = t_common.init_repo_rdf4j(url, "federated_micro_t1_nl_eq")

        adaptor = XmlModelAdaptor.from_folder(this_dir.parent / "data" / "micro")
        adaptor.adapt(eq_client.service_cfg.url)

        logger.info("Number of quads: %d", len(adaptor.graph))
        tpsvssh_ctx = adaptor.tpsvssh_contexts()
        tpsvssh = adaptor.nq_bytes(tpsvssh_ctx)

        eq_ctx = adaptor.eq_contexts()
        remaining = adaptor.nq_bytes(eq_ctx)

        logger.info("Federated micro model stats: EQ bytes: %d, TP/SH/SSH bytes: %d", len(remaining), len(tpsvssh))
        tpsvssh_client.upload_rdf(tpsvssh, "n-quads")
        eq_client.upload_rdf(remaining, "n-quads")

        m_cfg = ModelConfig(
            system_state_repo=tpsvssh_client.service_cfg.url + ",infer=false",
            eq_repo=eq_client.service_cfg.url + ",infer=false",
        )
        model = get_federated_cim_model(eq_client, tpsvssh_client, m_cfg)
    except Exception:
        logger.exception("Failed to get federated cim model")
    return t_common.ModelTest(model, name="micro-federated")


def upload_micro_model(client: GraphDBClient) -> None:
    adaptor = XmlModelAdaptor.from_folder(this_dir.parent / "data" / "micro")
    adaptor.adapt(client.service_cfg.url)
    graph = "<http://mygraph.com/demo/1/1>"
    client.upload_rdf(adaptor.nq_bytes(), "n-quads", {"context": graph})


@functools.lru_cache
def micro_t1_nl() -> t_common.ModelTest:
    """Micro model in RDF4J. It is cached, so multiple calls with the same url returns the same model."""
    model = None
    try:
        url = t_common.rdf4j_url()
        client = t_common.init_repo_rdf4j(url, "micro_t1_nl")
        upload_micro_model(client)
        model = SingleClientModel(client)
    except Exception:
        logger.exception("Failed to get single client model")
    return t_common.ModelTest(model, name="micro-combined")


@functools.lru_cache
def small_grid_model(url: str, api: RestApi) -> t_common.ModelTest:
    def bg_http(url: str, name: str) -> GraphDBClient:
        return new_repo_blazegraph(url, name, "http")

    model = None
    test_folder = Path(__file__).parent.parent / "data" / "small"
    try:
        init_func = t_common.init_repo_rdf4j if api == RestApi.RDF4J else bg_http
        eq_client = init_func(url, "smallgrid_eq")
        adaptor = XmlModelAdaptor.from_folder(test_folder)
        adaptor.adapt(eq_client.service_cfg.url)
        tpsvssh_client = init_func(url, "smallgrid_tpsvssh")
        tpsvssh_ctx = adaptor.tpsvssh_contexts()
        tpsvssh_client.upload_rdf(adaptor.nq_bytes(tpsvssh_ctx), "n-quads")

        remaining = adaptor.nq_bytes([ctx for ctx in adaptor.graph.contexts() if ctx not in tpsvssh_ctx])

        eq_client.upload_rdf(remaining, "n-quads")

        m_cfg = ModelConfig(
            system_state_repo=tpsvssh_client.service_cfg.url,
            eq_repo=eq_client.service_cfg.url,
        )
        model = get_federated_cim_model(eq_client, tpsvssh_client, m_cfg)
    except Exception:
        logger.exception("Failed to get federated model")
    return t_common.ModelTest(model, name="small-grid-federated")


def micro_models() -> list[t_common.ModelTest]:
    return [micro_t1_nl(), federated_micro_t1()]


def smallgrid_models() -> list[t_common.ModelTest]:
    rdfj4_model = small_grid_model(t_common.rdf4j_url(), RestApi.RDF4J)
    return [rdfj4_model]
